@@ -1,25 +1,28 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
 # for license details.
 """
 """
-from . import (_parameter, _block, _state, _model,
-              _metric, evaluation_tools, _variable)
-from ._variable import *
+from ._utils import *
+from ._feature import *
 from ._parameter import *
-from ._block import *
+from ._prediction import *
 from ._state import *
 from ._model import *
 from ._metric import *
+from . import (_parameter, _prediction, _state, _model,
+              _metric, evaluation_tools, _feature,
+              _utils)
 
 __all__ = ('evaluation_tools',
-           *_variable.__all__,
+           *_feature.__all__,
            *_parameter.__all__,
+           *_prediction.__all__,
            *_metric.__all__,
-           *_block.__all__,
            *_state.__all__,
-           *_model.__all__)
\ No newline at end of file
+           *_model.__all__,
+           *_utils.__all__)
\ No newline at end of file
