@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -25,10 +25,10 @@ class MassBalance(Unit):
 
     Parameters
     ----------
-    ins : stream
+    ins : 
         Inlet stream. Doesn't actually affect mass balance. It's just to
         show the position in the process.
-    outs : stream
+    outs : 
         Outlet stream. Doesn't actually affect mass balance. It's just to
         show the position in the process.
     chemical_IDs : tuple[str]
@@ -94,13 +94,12 @@ class MassBalance(Unit):
     >>> sys.simulate();
     >>> target.show()
     Stream: target
-     phase: 'l', T: 298.15 K, P: 101325 Pa
-     flow (kmol/hr): Water    500
-                     Ethanol  500
+    phase: 'l', T: 298.15 K, P: 101325 Pa
+    flow (kmol/hr): Water    500
+                    Ethanol  500
     
     """
     _graphics = process_specification_graphics
-    _N_heat_utilities = 0
     _N_ins = _N_outs = 1
 
     def __init__(self, ID='', ins=None, outs=(), thermo=None,
@@ -188,4 +187,4 @@ class MassBalance(Unit):
 #             raise ValueError(f"Type must be 'T', 'V' or 'F', not '{Type}'")
             
         
-        
\ No newline at end of file
+        
