@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -27,13 +27,22 @@ class DiagramOnlyUnit(Unit, isabstract=True):
     
     def _register(self, ID): 
         self.ID = self._ID = ID
+        
+    def _get_tooltip_string(self, *args, **kwargs): return ' '
+
     
 class DiagramOnlySystemUnit(DiagramOnlyUnit, isabstract=True):
     """Dummy unit for displaying a system as a unit."""
     line = 'System'
     _graphics = system_unit
+    
+    def _get_tooltip_string(self, *args, **kwargs): return ' '
+
 
 class DiagramOnlyStreamUnit(DiagramOnlyUnit, isabstract=True):
     """Dummy unit for displaying a streams as a unit."""
     line = ''
-    _graphics = stream_unit
\ No newline at end of file
+    _N_ins = _N_outs = 0
+    _graphics = stream_unit
+    
+    def _get_tooltip_string(self, *args, **kwargs): return ' '
\ No newline at end of file
