@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -19,21 +19,20 @@ class EnzymeTreatment(MixTank):
     
     def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
                  vessel_type='Conventional', tau=1.0, V_wf=0.9,
-                 vessel_material='Stainless steel', T):
+                 vessel_material='Stainless steel', T=None):
         MixTank.__init__(self, ID, ins, outs, thermo, vessel_type=vessel_type,
                          tau=tau, V_wf=V_wf, vessel_material=vessel_material)
-        self.T = T #: Operating temperature
-        self.heat_exchanger = HXutility(None, None, None, T=T)
-        self.heat_utilities = self.heat_exchanger.heat_utilities
+        self.T = T #: Operating temperature (if any)
+        self.heat_exchanger = HXutility(None, None, None, T=T, thermo=thermo)
     
     def _run(self):
-        feed = self.ins[0]
         out = self.outs[0]
-        out.mol[:] = self.mol_in
-        out.phase = feed.phase
-        out.P = feed.P
-        out.T = self.T
+        if self.T is None:
+            out.mix_from(self.ins)
+        else:
+            out.mix_from(self.ins, energy_balance=False)
+            out.T = self.T
         
     def _design(self):
         super()._design()
-        self.heat_exchanger.simulate_as_auxiliary_exchanger(self.Hnet, self.outs[0])
\ No newline at end of file
+        self.heat_exchanger.simulate_as_auxiliary_exchanger(self.ins, self.outs, self.Hnet)
\ No newline at end of file
