@@ -15,6 +15,7 @@ from . import design_tools as design
 from .splitting import Splitter
 from .heat_exchange import HX, HXutility
 from .._graphics import vertical_vessel_graphics
+import biosteam as bst
 
 exp = np.exp
 ln = np.log
@@ -39,9 +40,9 @@ class Flash(design.PressureVessel, Unit):
 
     Parameters
     ----------
-    ins : stream
+    ins : 
         Inlet fluid.
-    outs : stream sequence
+    outs : 
         * [0] Vapor product
         * [1] Liquid product
     P=None : float
@@ -99,7 +100,7 @@ class Flash(design.PressureVessel, Unit):
     Flash: F1
     ins...
     [0] feed
-        phase: 'l', T: 100.7 degC, P: 1 atm
+        phase: 'l', T: 100.67 degC, P: 1 atm
         flow (kmol/hr): Water     1e+03
                         Glycerol  300
     outs...
@@ -112,20 +113,21 @@ class Flash(design.PressureVessel, Unit):
         flow (kmol/hr): Water     42.4
                         Glycerol  298
     >>> F1.results()
-    Flash                                             Units            F1
-    Medium pressure steam Duty                        kJ/hr      4.78e+07
-                          Flow                      kmol/hr      1.32e+03
-                          Cost                       USD/hr           364
-    Design                Vessel type                            Vertical
-                          Length                         ft          15.5
-                          Diameter                       ft           8.5
-                          Weight                         lb      9.57e+03
-                          Wall thickness                 in         0.438
-                          Vessel material                    Carbon steel
-    Purchase cost         Vertical pressure vessel      USD      6.02e+04
-                          Heat exchanger                USD      4.23e+04
-    Total purchase cost                                 USD      1.02e+05
-    Utility cost                                     USD/hr           364
+    Flash                                                   Units            F1
+    Medium pressure steam Duty                              kJ/hr      4.81e+07
+                          Flow                            kmol/hr      1.33e+03
+                          Cost                             USD/hr           366
+    Design                Vessel type                                Horizontal
+                          Length                               ft          8.46
+                          Diameter                             ft           5.5
+                          Weight                               lb      2.51e+03
+                          Wall thickness                       in         0.312
+                          Vessel material                          Carbon steel
+    Purchase cost         Horizontal pressure vessel          USD      1.47e+04
+                          Platform and ladders                USD      3.22e+03
+                          Heat exchanger - Floating head      USD      4.48e+04
+    Total purchase cost                                       USD      6.26e+04
+    Utility cost                                           USD/hr           366
 
 
     References
@@ -144,18 +146,25 @@ class Flash(design.PressureVessel, Unit):
         Cost Accounting and Capital Cost Estimation (Chapter 16)
     
     """
-    auxiliary_unit_names = ('heat_exchanger',)
-    _units = {'Vertical vessel weight': 'lb',
-              'Horizontal vessel weight': 'lb',
-              'Length': 'ft',
+    auxiliary_unit_names = ('heat_exchanger', 'vacuum_system')
+    _auxin_index = {
+        'heat_exchanger': 0
+    }
+    _units = {'Length': 'ft',
               'Diameter': 'ft',
               'Weight': 'lb',
-              'Wall thickness': 'in'}
-    _BM = {'Liquid-ring pump': 1.0,
-           **design.PressureVessel._BM}
+              'Wall thickness': 'in',
+              'Total volume': 'ft3'}
+    _max_agile_design = (
+        'Length',
+        'Diameter',
+        'Weight',
+        'Wall thickness',
+    )
+    _F_BM_default = {'Liquid-ring pump': 1.0,
+                     **design.PressureVessel._F_BM_default}
     _graphics = vertical_vessel_graphics 
     _N_outs = 2
-    _N_heat_utilities = 0
 
     def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
                  V=None, T=None, Q=None, P=None, y=None, x=None,
@@ -168,10 +177,7 @@ class Flash(design.PressureVessel, Unit):
                  surge_time=7.5,
                  has_mist_eliminator=False):
         Unit.__init__(self, ID, ins, outs, thermo)
-        self._multi_stream = ms = MultiStream(None, thermo=self.thermo)
-        self.heat_exchanger = hx = HXutility(None, None, ms, thermo=self.thermo) 
-        self.heat_utilities = hx.heat_utilities
-        hx._ins = self._ins
+        self._load_components()
         
         #: Enforced molar vapor fraction
         self.V = V
@@ -185,7 +191,7 @@ class Flash(design.PressureVessel, Unit):
         #: [array_like] Molar composition of liquid (for binary mixture)
         self.x = x
         
-        #: Duty (kJ/hr)
+        #: Enforced duty (kJ/hr)
         self.Q = Q
         
         #: Operating pressure (Pa)
@@ -215,6 +221,16 @@ class Flash(design.PressureVessel, Unit):
         #: [bool] True if using a mist eliminator pad
         self.has_mist_eliminator = has_mist_eliminator
         
+    def _load_components(self):
+        self._multi_stream = ms = MultiStream(None, thermo=self.thermo)
+        self.auxiliary(
+            'heat_exchanger', HXutility, ins=self.feed, outs=ms
+        )
+        
+    def reset_cache(self, isdynamic=None):
+        self._multi_stream.reset_cache()
+        self.heat_exchanger.reset_cache()
+        
     @property
     def P(self):
         """Operating pressure (Pa)."""
@@ -224,81 +240,79 @@ class Flash(design.PressureVessel, Unit):
         if P and P < 101325 and not self.power_utility:
             self.power_utility = PowerUtility()
         self._P = P
+
+    @property
+    def vapor(self):
+        """Outlet vapor stream (equivalent to outs[0])."""
+        return self._outs[0]
+    @vapor.setter
+    def vapor(self, vapor):
+        self._outs[0] = vapor
     
     @property
-    def Q(self):
-        """Enforced duty (kJ/hr)."""
-        return self._Q
-    @Q.setter
-    def Q(self, Q):
-        self._Q = Q
+    def liquid(self):
+        """Outlet liquid stream (equivalent to outs[1])."""
+        return self._outs[1]
+    @liquid.setter
+    def liquid(self, liquid):
+        self._outs[1] = liquid
 
     def _default_vessel_type(self):
         vap, liq = self.outs
         F_mass_vap = vap.F_mass
         F_mass_liq = liq.F_mass 
-        assert F_mass_liq, "no liquid effluent; flash vessel must have liquid to default vessel type"
-        return 'Vertical'if F_mass_vap / F_mass_liq > 0.1 else 'Horizontal'
+        return 'Vertical' if F_mass_vap / F_mass_liq >= 1 else 'Horizontal'
 
     def _run(self):
         separations.vle(self.ins[0], *self.outs, self.T, self.P, self.V, 
                         self.Q, self.x, self.y, self._multi_stream)
-
+            
+    def _size_flash_vessel(self):
+        vap, liq, *_ = self.outs
+        self.no_vessel_needed = vap.isempty() or liq.isempty()
+        if self.no_vessel_needed:
+            self.design_results.clear()
+        else:
+            vessel_type = self.vessel_type
+            if vessel_type == 'Vertical': 
+                args = self._vertical_vessel_pressure_diameter_and_length()
+            elif vessel_type == 'Horizontal': 
+                args = self._horizontal_vessel_pressure_diameter_and_length()
+            else: raise RuntimeError('unknown vessel type') # pragma: no cover
+            self.design_results.update(
+                self._vessel_design(*args)
+            )
+        
     def _design(self):
-        vessel_type = self.vessel_type
-        if vessel_type == 'Vertical': 
-            args = self._vertical_vessel_pressure_diameter_and_length()
-        elif vessel_type == 'Horizontal': 
-            args = self._horizontal_vessel_pressure_diameter_and_length()
-        else: raise RuntimeError('unknown vessel type') # pragma: no cover
-        self.heat_exchanger._summary()
-        self.design_results.update(
-            self._vessel_design(*args)
-        )
+        self._size_flash_vessel()
+        if self.Q == 0.:
+            self.heat_exchanger._setup() # Removes results
+        else:
+            self.heat_exchanger.simulate_as_auxiliary_exchanger(self.ins, self.outs, P=self.ins[0].P)
 
     def _cost(self):
         D = self.design_results
-        self.purchase_costs.update(
-            self._vessel_purchase_cost(D['Weight'], D['Diameter'], D['Length'])
-        )
-        self._cost_vacuum()
+        if not self.no_vessel_needed:
+            self.baseline_purchase_costs.update(
+                self._vessel_purchase_cost(D['Weight'], D['Diameter'], D['Length'])
+            )
+            self._cost_vacuum()
 
     def _cost_vacuum(self):
         P = self.P
-        if not P or P > 101320: return 
-        
-        Design = self.design_results
-        volume = 0.02832 * np.pi * Design['Length'] * (Design['Diameter']/2)**2
-        
-        # If vapor is condensed, assume vacuum system is after condenser
-        vapor = self.outs[0]
-        hx = vapor.sink
-        if isinstance(hx, HX):
-            index = hx.ins.index(vapor)
-            stream = hx.outs[index]
-            if isinstance(stream, MultiStream): # pragma: no cover
-                vapor = stream['g']
-                F_mass = vapor.F_mass
-                F_vol = vapor.F_vol
-            else:
-                if stream.phase == 'g': # pragma: no cover
-                    F_mass = stream.F_mass
-                    F_vol = stream.F_vol
-                else:
-                    F_mass = 0
-                    F_vol = 0
+        if not P or P > 101320: 
+            self.vacuum_system = None
         else:
-            F_mass = vapor.F_mass
-            F_vol = vapor.F_vol
-        
-        power, cost = design.compute_vacuum_system_power_and_cost(
-                          F_mass, F_vol, P, volume, self.vacuum_system_preference)
-        self.purchase_costs['Liquid-ring pump'] = cost
-        self.power_utility(power)
+            Design = self.design_results
+            R = Design['Diameter'] * 0.5
+            volume = 0.02832 * np.pi * Design['Length'] * R * R # Volume ft3 to m3
+            self.vacuum_system = bst.VacuumSystem(
+                self, self.vacuum_system_preference, vessel_volume=volume,
+            )
 
     def _design_parameters(self):
         # Retrieve run_args and properties
-        vap, liq = self._outs
+        vap, liq, *_ = self._outs
         rhov = vap.get_property('rho', 'lb/ft3')
         rhol = liq.get_property('rho', 'lb/ft3')
         P = liq.get_property('P', 'psi')  # Pressure (psi)
@@ -345,19 +359,19 @@ class Flash(design.PressureVessel, Unit):
         if P < 300:
             Hlll = 1.25
 
-        # Calculate the height from Hlll to Normal liq level, Hnll
+        # Calculate the height from Hlll to Normal liquid level, Hnll
         Hh = Vh/(pi/4.0*Dvd**2)
         if Hh < 1.0: Hh = 1.0
 
-        # Calculate the height from Hnll to  High liq level, Hhll
+        # Calculate the height from Hnll to High liquid level, Hhll
         Hs = Vs/(pi/4.0*Dvd**2)
         if Hs < 0.5: Hs = 0.5
 
         # Calculate dN
-        Qm = Qll + Qv
-        lamda = Qll/Qm
-        rhoM = rhol*lamda + rhov*(1-lamda)
-        dN = (4*Qm/(pi*60.0/(rhoM**0.5)))**0.5
+        Qm = Qll / 60 + Qv
+        lamda = Qll / 60 / Qm
+        rhoM = rhol * lamda + rhov * (1 - lamda)
+        dN = (4*Qm / (pi * 60.0 / (rhoM**0.5)))**0.5
         dN = design.ceil_half_step(dN)
 
         # Calculate Hlin, assume with inlet diverter
@@ -491,11 +505,8 @@ class SplitFlash(Flash):
                  surge_time=7.5,
                  has_mist_eliminator=False):
         Splitter.__init__(self, ID, ins, outs, thermo, split=split, order=order)
-        self._multi_stream = ms = MultiStream(None, thermo=self.thermo)
+        self._load_components()
         
-        #: [HXutility] Heat exchanger if needed.
-        self.heat_exchanger = hx = HXutility(None, None, ms, thermo=self.thermo)
-        hx._ins = self._ins
         self.T = T #: Operating temperature (K)
         self.P = P #: Operating pressure (Pa)
         self.Q = Q #: Duty (kJ/hr)
@@ -531,29 +542,25 @@ class SplitFlash(Flash):
         top, bot = self.outs
         feed, = self.ins
         feed_mol = feed.mol
-        top.mol[:] = top_mol = feed_mol * self.split
-        bot.mol[:] = feed_mol - top_mol
         top.phase = 'g'
         bot.phase = 'l'
+        top.mol[:] = top_mol = feed_mol * self.split
+        bot.mol[:] = feed_mol - top_mol
         bot.T = top.T = self.T
         bot.P = top.P = self.P
 
     def _design(self):
-        self.heat_exchanger.outs[0] = ms = self._multi_stream
-        ms.mix_from(self.outs)
+        self.heat_exchanger.simulate_as_auxiliary_exchanger(self.ins, self.outs, vle=False)
         super()._design()
     
 # TODO: Remove this in favor of partition coefficients
 class RatioFlash(Flash):
-    _N_heat_utilities = 1
 
     def __init__(self, ID='', ins=None, outs=(), *,
                  K_chemicals, Ks, top_solvents=(), top_split=(),
                  bot_solvents=(), bot_split=()):
         Unit.__init__(self, ID, ins, outs)
-        self._multi_stream = ms = MultiStream(None, thermo=self.thermo)
-        #: [HXutility] Heat exchanger if needed.
-        self.heat_exchanger = HXutility(None, None, ms, thermo=self.thermo)
+        self._load_components()
         self.K_chemicals = K_chemicals
         self.Ks = Ks
         self.top_solvents = top_solvents
@@ -565,30 +572,44 @@ class RatioFlash(Flash):
         feed = self.ins[0]
         top, bot = self.outs
         indices = self.chemicals.get_index
-        K_index = indices(self.K_chemicals)
-        top_index = indices(self.top_solvents)
-        bot_index = indices(self.bot_solvents)
+        def flattend(indices, split):
+            flat_index = []
+            flat_split = []
+            integer = int
+            isa = isinstance
+            for i, j in zip(indices, split):
+                if isa(i, integer): 
+                    flat_index.append(i)
+                    flat_split.append(j)
+                else:
+                    flat_index.extend(i)
+                    flat_split.extend([j] * len(i))
+            return flat_index, np.array(flat_split)
+        
+        K_index, Ks = flattend(indices(self.K_chemicals), self.Ks)
+        top_index, top_split = flattend(indices(self.top_solvents), self.top_split)
+        bot_index, bot_split = flattend(indices(self.bot_solvents), self.bot_split)
         top_mol = top.mol; bot_mol = bot.mol; feed_mol = feed.mol
-        top_mol[top_index] = feed_mol[top_index]*self.top_split
-        bot_mol[top_index] = feed_mol[top_index]-top_mol[top_index]
-        bot_mol[bot_index] = feed_mol[bot_index]*self.bot_split
-        top_mol[bot_index] = feed_mol[bot_index]-bot_mol[bot_index]
+        top_mol[top_index] = feed_mol[top_index] * top_split
+        bot_mol[top_index] = feed_mol[top_index] - top_mol[top_index]
+        bot_mol[bot_index] = feed_mol[bot_index] * bot_split
+        top_mol[bot_index] = feed_mol[bot_index] - bot_mol[bot_index]
         topnet = top_mol[top_index].sum()
         botnet = bot_mol[bot_index].sum()
         molnet = topnet+botnet
-        top_mol[K_index] = self.Ks * topnet * feed_mol[K_index] / molnet  # solvent * mol ratio
+        top_mol[K_index] = Ks * topnet * feed_mol[K_index] / molnet  # solvent * mol ratio
         bot_mol[K_index] = feed_mol[K_index] - top_mol[K_index]
         top.T, top.P = feed.T, feed.P
         bot.T, bot.P = feed.T, feed.P
 
     def _design(self): # pragma: no cover
-        self.heat_exchanger.outs[0] = ms = self._multi_stream
-        ms.mix_from(self.outs)
+        self.heat_exchanger.simulate_as_auxiliary_exchanger(self.ins, self.outs, vle=False)
         super()._design()
 
-# %% Single Component
+# %% Single Component for MultiEffectEvaporator
 
-class Evaporator_PQ(Unit):
+class Evaporator(Flash):
+    _ins_size_is_fixed = True
     _N_ins = 2
     _N_outs = 3
     @property
@@ -596,39 +617,60 @@ class Evaporator_PQ(Unit):
         return self._P
     @P.setter
     def P(self, P):
-        water = getattr(self.chemicals, '7732-18-5')
-        self._T = T = water.Tsat(P)
-        self._Hvap = water.Hvap(T)
-        self._P = P
+        if P is None: 
+            self._Hvap = self._P = self._T = None
+        else:
+            chemical = self.chemical
+            self._T = T = chemical.Tsat(P)
+            self._Hvap = chemical.Hvap(T)
+            self._P = P
     @property
     def T(self):
         return self._T
     @T.setter
     def T(self, T): # pragma: no cover
-        water = getattr(self.chemicals, '7732-18-5')
-        self._P = water.Psat(T)
-        self._Hvap = water.Hvap(T)
-        self._T = T
-    @property
-    def V(self):
-        return self._V
+        if T is None: 
+            self._Hvap = self._P = self._T = None
+        else:
+            chemical = self.chemical
+            self._P = chemical.Psat(T)
+            self._T = T
     
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, Q=0, P=101325):
-        super().__init__(ID, ins, outs, thermo)
-        self.Q = Q
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, 
+                 Q=None, V=None, P=101325, chemical='7732-18-5'):
+        super().__init__(ID, ins, outs, thermo, vessel_type='Vertical')
+        self.chemical = self.chemicals[chemical]
         self.P = P
-        self._V = None
+        self.Q = Q
+        self.V = V
+    
+    def _load_components(self): pass
     
     def _run(self):
         feed, utility_vapor = self.ins
         vapor, liquid, utility_liquid = self.outs
         
-        # Optional if Q also comes from condensing a side stream
-        Q = self.Q
-        if not utility_vapor.isempty():
+        u_given = not utility_vapor.isempty()
+        Q_given = self.Q is not None
+        V_given = self.V is not None
+        N_specs = u_given + Q_given + V_given
+        if N_specs > 1:
+            raise RuntimeError('can specify at most one of the following: Q, V, or utility vapor')
+        
+        if Q_given:
+            Q = self.Q
+        elif u_given:
+            # Q can come from condensing a side stream
             utility_liquid.copy_like(utility_vapor)
             utility_liquid.phase = 'l'
-            Q += utility_vapor.Hvap
+            Q = utility_vapor.Hvap
+            Q_given = True
+        elif V_given:
+            V = self.V
+        else:
+            # Default to no heat transfer
+            Q = 0
+            Q_given = True
         feed_H = feed.H
     
         # Set exit conditions
@@ -637,68 +679,38 @@ class Evaporator_PQ(Unit):
         liquid.phase = 'l'
         vapor.phase = 'g'
         liquid.copy_flow(feed)
+        chemical_ID = self.chemical.ID
         
-        # Energy balance to find vapor fraction
-        f = feed.imol['7732-18-5']
-        H = feed_H + Q - liquid.H
-        if f:
-            V = H/(f * self._Hvap)
-            if V < 0: 
+        if Q_given:
+            # Energy balance to find vapor fraction
+            f = feed.imol[chemical_ID]
+            H = feed_H + Q
+            if f:
+                Hvap = f * self._Hvap
+                dH = (H - liquid.H)
+                V = dH / Hvap
+                if V < 0: 
+                    vapor.imol[chemical_ID] = 0
+                    liquid.imol[chemical_ID] = f
+                    liquid.H = H
+                elif V > 1:
+                    vapor.imol[chemical_ID] = f
+                    liquid.imol[chemical_ID] = 0
+                    utility_liquid.vle(H=utility_vapor.H - Hvap, P=utility_vapor.P)
+                else:
+                    vapor.imol[chemical_ID] = f * V
+                    liquid.imol[chemical_ID] = (1 - V) * f
+            else:
                 V = 0
-            elif V > 1:
-                V = 1
-        else:
-            V = 0
-        evaporated = f * V
-        vapor.imol['7732-18-5'] = evaporated
-        liquid.imol['7732-18-5'] = (1-V)*f
+                Q = 0
+        elif V_given:
+            # Specify fraction evaporated and compute Q
+            V = self.V
+            chemical_mol = feed.imol[chemical_ID]
+            liquid.copy_flow(feed)
+            liquid.imol[chemical_ID] = (1 - V) * chemical_mol
+            vapor.imol[chemical_ID] = V * chemical_mol
+            Q = liquid.H + vapor.H - feed.H
+        
         self.design_results['Heat transfer'] = Q
-        self._V = V
-
-
-class Evaporator_PV(Unit):
-    _N_heat_utilities = 0
-    _N_outs = 2
-    
-    @property
-    def P(self):
-        return self._P
-    @P.setter
-    def P(self, P):
-        water = getattr(self.chemicals, '7732-18-5')
-        self._T = water.Tsat(P)
-        self._P = P
-    @property
-    def T(self):
-        return self._T
-    @T.setter
-    def T(self, T): # pragma: no cover
-        water = getattr(self.chemicals, '7732-18-5')
-        self._P = water.Psat(T)
-        self._T = T
-    
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, V=0.5, P=101325):
-        super().__init__(ID, ins, outs, thermo)
-        self._multi_stream = ms = MultiStream(None, thermo=self.thermo)
-        self.heat_exchanger = hx = HXutility(None, None, ms, thermo=self.thermo) 
-        self.heat_utilities = hx.heat_utilities
-        self.V = V
-        self.P = P
-        hx._ins = self._ins
-
-    def _run(self):
-        feed = self.ins[0]
-        vapor, liquid = self.outs
-        vapor.phase = 'g'
-        vapor.T = liquid.T = self.T
-        vapor.P = liquid.P = self.P
-        H2O_index = self.chemicals.index('7732-18-5')
-        water_mol = feed.mol[H2O_index]
-        vapor.mol[H2O_index] = self.V * water_mol
-        liquid_mol = liquid.mol
-        liquid_mol[:] = feed.mol
-        liquid_mol[H2O_index] = (1-self.V) * water_mol
-        ms = self._multi_stream
-        self.design_results['Heat transfer'] = self.heat_exchanger.Q = (vapor.H + liquid.H) - feed.H
-        ms['g'].copy_like(vapor)
-        ms['l'].copy_like(liquid)
\ No newline at end of file
+        
