@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -8,10 +8,13 @@
 """
 """
 import numpy as np
-from .design_tools.mechanical import (calculate_NPSH,
-                                      pump_efficiency, 
-                                      nearest_NEMA_motor_size,
-                                      nema_sizes_hp)
+from .design_tools.mechanical import (
+    calculate_NPSH,
+    pump_efficiency, 
+    nearest_NEMA_motor_size,
+    nema_sizes_hp,
+    electric_motor_cost
+)
 from .design_tools.specification_factors import (
     pump_material_factors,
     pump_centrifugal_factors)
@@ -42,19 +45,19 @@ class Pump(Unit):
 
     Parameters
     ----------
-    ins : stream
+    ins : 
         Inlet.
-    outs : stream
+    outs : 
         Outlet.
     P=101325 : float, optional
         Pressure of output stream (Pa). If the pressure of the outlet is 
-        the same as the inlet, pump is design to increase of pressure
+        the same as the inlet, pump is designed to increase of pressure
         to `dP_design`.
     pump_type='Default' : str
         If 'Default', a pump type is selected based on heuristics.
     material : str, optional
         Contruction material of pump. Defaults to 'Cast iron'.
-    dP_design=405300 : float
+    dP_design=101325 : float
         Design pressure change for pump.
     ignore_NPSH=True : bool
         Whether to take into consideration NPSH in the selection of a
@@ -62,7 +65,7 @@ class Pump(Unit):
     
     Notes
     -----
-    Default pump selection and design and cost algorithms are based on [0]_.
+    Default pump selection and design and cost algorithms are based on [1]_.
     
     Examples
     --------
@@ -85,36 +88,36 @@ class Pump(Unit):
         phase: 'l', T: 350 K, P: 200000 Pa
         flow (kmol/hr): Water  200
     >>> P1.results()
-    Pump                               Units           P1
-    Power               Rate              kW        0.289
-                        Cost          USD/hr       0.0226
-    Design              Ideal power       hp        0.136
-                        Flow rate        gpm         16.3
-                        Efficiency                  0.352
-                        Actual power                0.387
-                        Pump power                    0.5
-                        N                               1
-                        Head              ft         96.5
-                        Type                  Centrifugal
-    Purchase cost       Pump             USD     4.37e+03
-                        Motor            USD          311
-    Total purchase cost                  USD     4.68e+03
-    Utility cost                      USD/hr       0.0226
+    Pump                              Units           P1
+    Electricity         Power            kW        0.288
+                        Cost         USD/hr       0.0225
+    Design              Type                 Centrifugal
+                        Ideal power      hp        0.136
+                        Flow rate       gpm         16.3
+                        Efficiency                 0.352
+                        Power            hp        0.386
+                        Head             ft         96.3
+                        Motor size       hp          0.5
+    Purchase cost       Pump            USD     4.37e+03
+                        Motor           USD          273
+    Total purchase cost                 USD     4.64e+03
+    Utility cost                     USD/hr       0.0225
     
     References
     ----------
-    .. [0] Seider, Warren D., et al. (2017). "Cost Accounting and Capital Cost
+    .. [1] Seider, Warren D., et al. (2017). "Cost Accounting and Capital Cost
         Estimation". In Product and Process Design Principles: Synthesis,
         Analysis, and Evaluation (pp. 450-455). New York: Wiley.
     
     """
     _units = {'Ideal power': 'hp',
+              'Motor size': 'hp',
               'Power': 'hp',
               'Head': 'ft',
               'NPSH': 'ft',
               'Flow rate': 'gpm'}
-    _BM = {'Pump': 3.3,
-           'Motor': 3.3}
+    _F_BM_default = {'Pump': 3.3,
+                     'Motor': 3.3}
 
     @property
     def pump_type(self):
@@ -133,17 +136,17 @@ class Pump(Unit):
     @material.setter    
     def material(self, material):
         try:
-            self._F_M = pump_material_factors[material]
+            self.F_M['Pump'] = pump_material_factors[material]
         except KeyError:
             raise ValueError("material must be one of the following: "
                             f"{', '.join(pump_material_factors)}")
-        self._F_Mstr = material  
+        self._material = material  
         
     def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
-                 P=101325,
+                 P=None,
                  pump_type='Default',
                  material='Cast iron',
-                 dP_design=405300,
+                 dP_design=101325,
                  ignore_NPSH=True):
         Unit.__init__(self, ID, ins, outs, thermo)
         self.P = P
@@ -152,16 +155,11 @@ class Pump(Unit):
         self.dP_design = dP_design
         self.ignore_NPSH = ignore_NPSH
     
-    def _setup(self):
-        s_in, = self.ins
-        s_out, = self.outs
-        s_out.P = self.P
-    
     def _run(self):
         s_in, = self.ins
         s_out, = self.outs
-        s_out.T = s_in.T
-        if self.P < s_in.P: s_out.P = s_in.P 
+        s_out.copy_like(s_in)
+        if self.P: s_out.P = self.P 
     
     def _design(self):
         Design = self.design_results
@@ -179,91 +177,86 @@ class Pump(Unit):
         if dP < 1: dP = self.dP_design
         power_ideal = Qi*dP*3.725e-7 # hp
         q = Qi*4.403 # gpm
-        # Assume 100% efficiency to estimate the number of pumps
-        # Note: i subscript refers to an individual pump
-        N_min = ceil(q / 5000)
+        N = 1
+        
+        # Note:
+        # A pump motor can have maximally `max_hp` power, so need to first solve
+        # for how many pumps with maximally `max_hp` are needed. 
+        # Assume 100% efficiency to estimate the number of pumps.
+        # Additionally, pump types have their own maximum size.
         N_guess = -1
-        N = max(ceil(power_ideal / max_hp), N_min)
-        while N != N_guess: 
+        while N != N_guess:
             N_guess = N
             power_ideal_i = power_ideal / N_guess
             q_i = q / N_guess
-            # Recalculate number of pumps given true efficiency
-            efficiency = pump_efficiency(q_i, power_ideal_i)
-            power = power_ideal / efficiency
-            N = max(ceil(power / max_hp), N_min)
-        Design['Ideal power'] = power_ideal_i
-        Design['Flow rate'] = q_i
-        Design['Efficiency'] = efficiency = pump_efficiency(q_i, power_ideal_i) 
-        Design['Actual power'] = power_i = power_ideal_i/efficiency
-        Design['Pump power'] = nearest_NEMA_motor_size(power_i)
-        Design['N'] = N
-        Design['Head'] = head = power / mass*897806 # ft
-        # Note that:
-        # head = power / (mass * gravity)
-        # head [ft] = power[hp]/mass[kg/hr]/9.81[m/s^2] * conversion_factor
-        # and 897806 = (conversion_factor/9.81)
-        
-        if self.ignore_NPSH:
-            NPSH_satisfied = True
-        else:
-            Design['NPSH'] = NPSH = calculate_NPSH(Pi, si.P_vapor, si.rho)
-            NPSH_satisfied = NPSH > 1.52
-        
-        # Get type
-        pump_type = self.pump_type
-        if pump_type == 'Default':
-            if (0.00278 <= q_i <= 5000
-                and 15.24 <= head <= 3200
-                and nu <= 0.00002
-                and NPSH_satisfied):
-                pump_type = 'Centrifugal'
-            elif (q_i <= 1500
-                  and head <= 914.4
-                  and 0.00001 <= nu <= 0.252):
-                pump_type = 'Gear'
-            elif (head <= 20000
-                  and q_i <= 500
-                  and power <= 200
-                  and nu <= 0.01):
-                pump_type = 'MeteringPlunger'
+            efficiency = pump_efficiency(q_i, power_ideal_i) 
+            power_i = power_ideal_i / efficiency
+            head_i = N * power_i / mass * 897806 # ft
+            # Note that:
+            # head = power / (mass * gravity)
+            # head [ft] = power[hp]/mass[kg/hr]/9.81[m/s^2] * conversion_factor
+            # and 897806 = (conversion_factor/9.81)
+            
+            if self.ignore_NPSH:
+                NPSH_satisfied = True
             else:
-                NPSH = calculate_NPSH(Pi, si.P_vapor, si.rho)
-                warn(f'no pump type available at current power '
-                     f'({power:.3g} hp), flow rate ({q_i:.3g} gpm), and head '
-                     f'({head:.3g} ft), kinematic viscosity ({nu:.3g} m2/s), '
-                     f'and NPSH ({NPSH:.3g} ft); assuming centrigugal pump')
-                pump_type = 'Centrifugal'
-                
+                Design['NPSH'] = NPSH = calculate_NPSH(Pi, si.P_vapor, si.rho)
+                NPSH_satisfied = NPSH > 1.52
+            
+            pump_type = self.pump_type
+            if pump_type == 'Default':
+                if (15.24 <= head_i <= 3200
+                    and nu <= 0.00002
+                    and NPSH_satisfied):
+                    pump_type = 'Centrifugal'
+                elif (head_i <= 914.4
+                      and 0.00001 <= nu <= 0.252):
+                    N = max(ceil(q_i / 1500), N)
+                    pump_type = 'Gear'
+                elif (head_i <= 20000
+                      and power_i <= 200
+                      and nu <= 0.01):
+                    N = max(ceil(q_i / 500), N)
+                    pump_type = 'MeteringPlunger'
+                else:
+                    NPSH = calculate_NPSH(Pi, si.P_vapor, si.rho)
+                    warn(f'{repr(self)} no pump type available at current power '
+                         f'({power_i:.3g} hp), head ({head_i:.3g} ft), kinematic '
+                         f'viscosity ({nu:.3g} m2/s), and NPSH ({NPSH:.3g} ft); '
+                          'assuming centrigugal pump',
+                         RuntimeWarning)
+                    pump_type = 'Centrifugal'
         Design['Type'] = pump_type
-        self.power_utility(power/N/1.341) # Set power in kW
+        Design['Ideal power'] = power_ideal_i = power_ideal / N
+        Design['Flow rate'] = q_i = q / N_guess
+        Design['Efficiency'] = pump_efficiency(q_i, power_ideal_i) 
+        Design['Power'] = power_i = power_ideal_i / efficiency
+        Design['Head'] = N * power_i / mass * 897806 # ft
+        Design['Motor size'] = motor_rating = nearest_NEMA_motor_size(power_i)
+        self.add_power_utility(power_i / 1.341) # Add power for individual pump in kW
+        self.parallel['self'] = N # BioSTEAM will multiply all costs (both capital and utility) by this number
+        self.parallel['Motor'] = N * ceil(power_i / motor_rating) # Multiple motors per pump is posible
     
     def _cost(self):
-        if self.ins[0].isempty(): 
-            self.purchase_costs.clear()
-            return
-        
-        # Parameters
         Design = self.design_results
-        Cost = self.purchase_costs
+        if not Design: return
+        Cost = self.baseline_purchase_costs
         pump_type = Design['Type']
         q = Design['Flow rate']
         h = Design['Head']
-        p = Design['Pump power']
-        F_M = self._F_M
+        p = Design['Power']
         I = bst.CE/567
-        lnp = ln(p)
         
         # TODO: Add cost equation for small pumps
         # Head and flow rate is too small, so make conservative estimate on cost
         if q < 50: q = 50
         if h < 50: h = 50
+        F_T = 1 # Assumption
         
         # Cost pump
         if 'Centrifugal' in pump_type:
             # Find pump factor
             F_Tdict = pump_centrifugal_factors
-            F_T = 1 # Assumption
             if p < 75 and 50 <= q <= 900 and 50 <= h <= 400:
                 F_T = F_Tdict['VSC3600']
             elif p < 200 and 50 <= q <= 3500 and 50 <= h <= 2000:
@@ -294,25 +287,24 @@ class Pump(Unit):
             lnS = ln(S_new)
             Cb = exp(12.1656-1.1448*lnS+0.0862*lnS**2)
             Cb *= S/S_new
-            Cost['Pump'] = F_M*F_T*Cb*I
+            Cost['Pump'] = Cb*I
         elif pump_type == 'Gear':
             q_new = q if q > 50 else 50
             lnq = ln(q_new)
             Cb = exp(8.2816 - 0.2918*lnq + 0.0743*lnq**2)
             Cb *= q/q_new
-            Cost['Pump'] = F_M*Cb*I
+            Cost['Pump'] = Cb*I
         elif pump_type == 'MeteringPlunger':
-            Cb = exp(7.9361 + 0.26986*lnp + 0.06718*lnp**2)
-            Cost['Pump'] = F_M*Cb*I
+            lnp = ln(p)
+            lnp2 = lnp * lnp
+            Cb = exp(7.9361 + 0.26986*lnp + 0.06718*lnp2)
+            Cost['Pump'] = Cb*I
+        
+        self.F_D['Pump'] = F_T
         
         # Cost electric motor
-        lnp2 = lnp**2
-        lnp3 = lnp2*lnp
-        lnp4 = lnp3*lnp
-        Cost['Motor'] = exp(5.9332 + 0.16829*lnp
-                            - 0.110056*lnp2 + 0.071413*lnp3
-                            - 0.0063788*lnp4)*I
+        Cost['Motor'] = electric_motor_cost(p)
     
         
         
-        
\ No newline at end of file
+        
