@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -25,9 +25,10 @@ class VentScrubber(Unit):
         water, vent_entry = self.ins
         vent_exit, bottoms = self.outs
         vent_exit.copy_like(vent_entry)
+        bottoms.empty()
         bottoms.copy_flow(vent_exit, self.gas,
-                         remove=True, exclude=True)
-        bottoms.mol[:] += water.mol
+                          remove=True, exclude=True)
+        bottoms.mix_from([bottoms, water], energy_balance=False)
         
     def _design(self):
-        self.design_results['Flow rate'] = self._outs[0].F_mass
\ No newline at end of file
+        self.design_results['Flow rate'] = self._ins[1].F_mass
\ No newline at end of file
