@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -18,7 +18,11 @@ __all__ = ('design',)
 def _design(self):
     D = self.design_results
     U = self._units
-    for i, j in self._design_basis_: D[i] = j(self, U[i])
+    for i, j, k in self._design_basis_: 
+        if k:
+            D[i] = j(self, U[i]) * self.system.operating_hours / 8760.
+        else:
+            D[i] = j(self, U[i])
 
 class DesignCenter:
     """
@@ -52,7 +56,7 @@ class DesignCenter:
         functions[name] = design_basis
         return design_basis
     
-    def __call__(self, name, units, fsize=None):    
+    def __call__(self, name, units):    
         """
         Return a Unit class decorator that adds a size/design requirement to the class.
         
@@ -62,14 +66,11 @@ class DesignCenter:
             Name of design item            
         units : str
             Units of measure of design item.            
-        fsize : function
-            Should return design item given the Unit object.
-            If None, defaults to function predefined for given name and units.
         
         """
-        return lambda cls: self.add_design_basis_to_cls(cls, name, units, fsize)
+        return lambda cls: self.add_design_basis_to_cls(cls, name, units)
     
-    def add_design_basis_to_cls(self, cls, name, units, fsize=None):
+    def add_design_basis_to_cls(self, cls, name, units, annual):
         """
         Add size/design requirement to class.
         
@@ -79,10 +80,7 @@ class DesignCenter:
         name : str
             Name of design item.        
         units : str
-            Units of measure of design item.        
-        fsize : function
-            Should return design item given the Unit object.
-            If None, defaults to function predefined for given name and units.
+            Units of measure of design item.
             
         Examples
         --------
@@ -90,7 +88,11 @@ class DesignCenter:
         :doc:`Unit decorators`
         
         """
-        f = fsize or self.design_basis_functions[name.capitalize()]
+        key = name.capitalize()
+        if key.startswith('Annual '):
+            key = key[7:].capitalize()
+            if annual is None: annual = True 
+        f = self.design_basis_functions[key]
         
         # Make sure design basis is not defined
         if name in cls._units:
@@ -100,9 +102,9 @@ class DesignCenter:
         
         # Add design basis
         if hasattr(cls, '_decorated_design'):
-            cls._design_basis_.append((name, f))
+            cls._design_basis_.append((name, f, annual))
         else:
-            cls._design_basis_ = [(name, f)]
+            cls._design_basis_ = [(name, f, annual)]
             cls._decorated_design = _design
         if '_design' not in cls.__dict__:
             cls._design = _design
@@ -137,8 +139,8 @@ H_units = stream_units_of_measure['H']
 
 @design.define
 def duty(self, units):
-    self._duty = duty = self.H_out - self.H_in
-    self.heat_utilities[0](duty, self.ins[0].T, self.outs[0].T)
+    duty = self.H_out - self.H_in
+    self.add_heat_utility(duty, self.ins[0].T, self.outs[0].T)
     return H_units.conversion_factor(units) * duty
     
 @design.define
