@@ -1,12 +1,13 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
 # for license details.
 """
 """
+from . import agitator
 from . import vacuum
 from . import pressure_vessel
 from . import flash_vessel_design
@@ -17,21 +18,26 @@ from . import column_design
 from . import heat_transfer
 from . import tank_design
 from . import geometry
+from . import mechanical
 from . import utils
 
-__all__ = (*cost_index.__all__,
-           *pressure_vessel.__all__,
-           *vacuum.__all__,
-           *flash_vessel_design.__all__,
-           *batch.__all__,
-           *specification_factors.__all__,
-           *column_design.__all__,
-           *heat_transfer.__all__,
-           *tank_design.__all__,
-           *utils.__all__,
-           *geometry.__all__,
+__all__ = (
+    *agitator.__all__,
+    *cost_index.__all__,
+    *pressure_vessel.__all__,
+    *vacuum.__all__,
+    *flash_vessel_design.__all__,
+    *batch.__all__,
+    *specification_factors.__all__,
+    *column_design.__all__,
+    *heat_transfer.__all__,
+    *tank_design.__all__,
+    *mechanical.__all__,
+    *utils.__all__,
+    *geometry.__all__,
 )
 
+from .agitator import *
 from .pressure_vessel import *
 from .specification_factors import *
 from .column_design import *
@@ -41,5 +47,6 @@ from .cost_index import *
 from .batch import *
 from .heat_transfer import *
 from .tank_design import *
+from .mechanical import *
 from .utils import *
 from .geometry import *
\ No newline at end of file
