@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -19,28 +19,31 @@ References
 
 """
 from numpy import log as ln, pi, exp, round
-from flexsolve import njitable
+from numba import njit
 import biosteam as bst
 __all__ = ('compute_horizontal_vessel_purchase_cost',
            'compute_vertical_vessel_purchase_cost',
+           'compute_horizontal_vessel_platform_and_ladders_purchase_cost',
+           'compute_vertical_vessel_platform_and_ladders_purchase_cost',
            'GTable', 'HNATable', 'ceil_half_step',
            'compute_vessel_weight_and_wall_thickness',
            'compute_Stokes_law_York_Demister_K_value')
 
-@njitable(cache=True)
-def compute_horizontal_vessel_purchase_cost(W, D, F_M):
+@njit(cache=True)
+def compute_horizontal_vessel_purchase_cost(W):
     """
     Return the purchase cost [Cp; in USD] of a horizontal vessel,
-    including the cost of platforms and ladders.
+    without the cost of the platform and ladders.
     
     Parameters
     ----------
     W : float
         Weight [lb].
-    D : float
-        Diameter [ft].
-    F_M : float
-        Vessel material factor.
+    
+    Examples
+    --------
+    >>> compute_horizontal_vessel_purchase_cost(W=1e3)
+    8857.9578
     
     Notes
     -----
@@ -49,28 +52,80 @@ def compute_horizontal_vessel_purchase_cost(W, D, F_M):
     Plant Cost Index, `biosteam.CE`.
     
     """
-    # C_v: Vessel cost
-    # C_pl: Platforms and ladders cost
-    C_v = exp(5.6336 - 0.4599*ln(W) + 0.00582*ln(W)**2)
-    C_pl = 2275*D**0.20294
-    return bst.CE/567 * (F_M * C_v + C_pl)
+    lnW = ln(W)
+    C_v = exp(5.6336 + 0.4599 * lnW + 0.00582 * lnW * lnW)
+    return bst.CE/567 * C_v
 
-@njitable(cache=True)
-def compute_vertical_vessel_purchase_cost(W, D, L, F_M):
+@njit(cache=True)
+def compute_horizontal_vessel_platform_and_ladders_purchase_cost(D):
+    """
+    Return the purchase cost [Cp; in USD] of the platform and ladders for a 
+    horizontal vessel.
+    
+    Parameters
+    ----------
+    D : float
+        Diameter [ft].
+    
+    Examples
+    --------
+    >>> compute_horizontal_vessel_platform_and_ladders_purchase_cost(D=4)
+    3016.7982
+    
+    Notes
+    -----
+    The purchase cost is given by [1]_. See source code for details.
+    The purchase cost is scaled according to BioSTEAM's Chemical
+    Plant Cost Index, `biosteam.CE`.
+    
+    """
+    C_pl = 2275.*D**0.20294
+    return bst.CE/567. * C_pl
+
+@njit(cache=True)
+def compute_vertical_vessel_purchase_cost(W):
     """
     Return the purchase cost [Cp; in USD] of a vertical vessel,
-    including the cost of platforms and ladders.
+    without the cost of the platform and ladders.
     
     Parameters
     ----------
     W : float
         Weight [lb].
+    
+    Examples
+    --------
+    >>> compute_vertical_vessel_purchase_cost(1e3)
+    13319.0892
+    
+    Notes
+    -----
+    The purchase cost is given by [1]_. See source code for details.
+    The purchase cost is scaled according to BioSTEAM's Chemical
+    Plant Cost Index, `biosteam.CE`.
+    
+    """
+    lnW = ln(W)
+    C_v = exp(7.1390 + 0.18255 * lnW + 0.02297 * lnW * lnW)
+    return bst.CE/567. * C_v
+
+@njit(cache=True)
+def compute_vertical_vessel_platform_and_ladders_purchase_cost(D, L):
+    """
+    Return the purchase cost [Cp; in USD] of the platform and ladders for a 
+    vertical vessel.
+    
+    Parameters
+    ----------
     D : float
         Diameter [ft].
     L : float
         Length [ft].
-    F_M : float
-        Vessel material factor.
+    
+    Examples
+    --------
+    >>> compute_vertical_vessel_platform_and_ladders_purchase_cost(3, 10)
+    4708.5321
     
     Notes
     -----
@@ -79,11 +134,8 @@ def compute_vertical_vessel_purchase_cost(W, D, L, F_M):
     Plant Cost Index, `biosteam.CE`.
     
     """
-    # C_v: Vessel cost
-    # C_pl: Platforms and ladders cost
-    C_v = exp(7.1390 + 0.18255*ln(W) + 0.02297*ln(W)**2)
     C_pl = 410*D**0.7396*L**0.70684
-    return bst.CE/567 * (F_M * C_v + C_pl)
+    return bst.CE/567 * C_pl
 
 def GTable(DRho, Hlr):
     """
@@ -191,7 +243,7 @@ def GTable(DRho, Hlr):
 
     return round(G, 2)
 
-@njitable(cache=True)
+@njit(cache=True)
 def HNATable(Type, X):
     """
     Table for cylindrical height and area conversions.
@@ -236,7 +288,7 @@ def HNATable(Type, X):
 
     return Y
 
-@njitable(cache=True)
+@njit(cache=True)
 def compute_vessel_weight_and_wall_thickness(P, D, L, rho_M, Je=0.85):
     """
     Return vessel weight and wall thickness.
@@ -254,6 +306,11 @@ def compute_vessel_weight_and_wall_thickness(P, D, L, rho_M, Je=0.85):
     Je: float
         Joint efficiency (1.0 for X-Rayed joints, 0.85 for thin carbon steel),
     
+    Examples
+    --------
+    >>> compute_vessel_weight_and_wall_thickness(14.7, 3, 10, 490)
+    (1116.83, 0.25)
+    
     Notes
     -----
     Equations are given by [2]_. See source code for details.
@@ -317,7 +374,7 @@ def compute_vessel_weight_and_wall_thickness(P, D, L, rho_M, Je=0.85):
     VW = round(VW, 2)
     return VW, ts
 
-@njitable(cache=True)
+@njit(cache=True)
 def compute_low_liq_level_height(Type, P, D):
     """
     Return the height of the lowest liquid level [Hlll; in ft]
@@ -358,7 +415,7 @@ def compute_low_liq_level_height(Type, P, D):
 
     return Hlll  # in ft
 
-@njitable(cache=True)
+@njit(cache=True)
 def compute_Stokes_law_York_Demister_K_value(P):
     """
     Return K-constant in Stoke's Law using the York-Demister equation.
@@ -367,7 +424,16 @@ def compute_Stokes_law_York_Demister_K_value(P):
     ----------
     P : float
         Pressure [psia].
-        
+    
+    Examples
+    --------
+    >>> compute_Stokes_law_York_Demister_K_value(14)
+    0.34409663
+    >>> compute_Stokes_law_York_Demister_K_value(20)
+    0.35
+    >>> compute_Stokes_law_York_Demister_K_value(125)
+    0.31894878
+    
     Notes
     -----
     Equations are given by [2]_. See source code for details.
@@ -379,11 +445,13 @@ def compute_Stokes_law_York_Demister_K_value(P):
         K = 0.35
     elif P > 40.0 and P <= 5500.0:
         K = 0.43 - 0.023*ln(P)
+    elif P < 0:
+        raise ValueError('invalid Pressure of over 5500 psia')
     else:
-        raise ValueError(f'invalid Pressure {P} psia')
+        raise ValueError('invalid Pressure of over 5500 psia')
     return K
 
-@njitable(cache=True)
+@njit(cache=True)
 def ceil_half_step(value):
     """Return value to the next highest 0.5 units"""
     intval = round(value)
