@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -10,7 +10,7 @@ General functional algorithms for the design of heat exchangers.
 
 """
 from numpy import log as ln
-from flexsolve import njitable
+from numba import njit
 
 __all__ = ('counter_current_heat_exchange',
            'heuristic_overall_heat_transfer_coefficient',
@@ -23,27 +23,77 @@ __all__ = ('counter_current_heat_exchange',
 
 # %% Functional heat exchanger
 
-def heat_exchange_to_condition(s_in, s_out, T, phase=None):
+def heat_exchange_to_condition(s_in, s_out, T=None, phase=None, 
+                               H_lim=None, f_equilibrium=None, heating=None):
     """
     Set the outlet stream condition and return duty required to achieve 
     said condition.
     """
+    H_lim_given = H_lim is not None
+    H_in = s_in.H
+    if H_lim_given:
+        if heating:
+            if H_in > H_lim: return 0.
+        else:
+            if H_in < H_lim: return 0.
     if phase:
         s_out.T = T
         s_out.phase = phase
+        if H_lim_given:
+            if heating:
+                if s_out.H > H_lim: s_out.H = H_lim
+            else:
+                if s_out.H < H_lim: s_out.H = H_lim
+    elif len(s_out.vle_chemicals) == 1 and heating is not None:
+        bp = s_in.bubble_point_at_P()
+        tol = 1e-3
+        dT = T - bp.T
+        s_out.T = T
+        if dT < -tol:
+            s_out.phase = 'l'
+            if H_lim_given:
+                if heating:
+                    if s_out.H > H_lim: s_out.H = H_lim
+                else:
+                    if s_out.H < H_lim: s_out.vle(H=H_lim , P=s_out.P)
+        elif dT > tol:
+            s_out.phase = 'g'
+            if H_lim_given:
+                if heating:
+                    if s_out.H > H_lim: s_out.vle(H=H_lim , P=s_out.P)
+                else:
+                    if s_out.H < H_lim: s_out.H = H_lim
+        else:
+            s_out.phase = 'g' if heating else 'l'
     else:
         s_out.vle(T=T, P=s_out.P)
-    return s_out.H - s_in.H
+        if H_lim_given:
+            if heating:
+                if s_out.H > H_lim: s_out.vle(H=H_lim , P=s_out.P)
+            else:
+                if s_out.H < H_lim: s_out.vle(H=H_lim , P=s_out.P)
+    
+    # Sanity check
+    if heating:
+        assert s_out.T > s_in.T - 0.01, "failed to solve temperature"
+        if s_out.T < s_in.T: s_out.T = s_in.T
+    else:
+        assert s_out.T <= s_in.T + 0.01, "failed to solve temperature"
+        if s_out.T > s_in.T: s_out.T = s_in.T
+    
+    return s_out.H - H_in
+    
 
 def counter_current_heat_exchange(s0_in, s1_in, s0_out, s1_out,
                                   dT, T_lim0=None, T_lim1=None,
-                                  phase0=None, phase1=None):
+                                  phase0=None, phase1=None,
+                                  H_lim0=None, H_lim1=None):
     """
     Allow outlet streams to exchange heat until either the given temperature 
     limits or the pinch temperature and return the total heat transfer 
     [Q; in kJ/hr].
     """
-    # Counter_current_heat_exchange_setup:
+    # Counter current heat exchange setup:
     # First find the hot inlet, cold inlet, hot outlet and cold outlet streams
     # along with the maximum temperature approaches for the hotside and the 
     # cold side.
@@ -54,6 +104,8 @@ def counter_current_heat_exchange(s0_in, s1_in, s0_out, s1_out,
         s_cold_out = s1_out
         T_lim_coldside = T_lim0
         T_lim_hotside = T_lim1
+        H_lim_coldside = H_lim0
+        H_lim_hotside = H_lim1
         phase_coldside = phase0
         phase_hotside = phase1
     else:
@@ -63,6 +115,8 @@ def counter_current_heat_exchange(s0_in, s1_in, s0_out, s1_out,
         s_hot_out = s1_out
         T_lim_hotside = T_lim0
         T_lim_coldside = T_lim1
+        H_lim_hotside = H_lim0
+        H_lim_coldside = H_lim1
         phase_hotside = phase0
         phase_coldside = phase1
     
@@ -70,13 +124,19 @@ def counter_current_heat_exchange(s0_in, s1_in, s0_out, s1_out,
     
     T_pinch_coldside = s_cold_in.T + dT
     if T_lim_coldside:
-        T_lim_coldside = max(T_pinch_coldside, T_lim_coldside) 
+        if T_lim_coldside > s_hot_in.T:
+            return 0. # No heat exchange
+        else:
+            T_lim_coldside = max(T_pinch_coldside, T_lim_coldside) 
     else:
         T_lim_coldside = T_pinch_coldside
     
     T_pinch_hotside = s_hot_in.T - dT
     if T_lim_hotside:
-        T_lim_hotside = min(T_pinch_hotside, T_lim_hotside) 
+        if T_lim_hotside < s_cold_in.T:
+            return 0. # No heat exchange
+        else:
+            T_lim_hotside = min(T_pinch_hotside, T_lim_hotside) 
     else:
         T_lim_hotside = T_pinch_hotside
        
@@ -85,14 +145,25 @@ def counter_current_heat_exchange(s0_in, s1_in, s0_out, s1_out,
        
     # Pinch on the cold side
     Q_hot_stream = heat_exchange_to_condition(s_hot_in, s_hot_out, 
-                                              T_lim_coldside, phase_coldside)
+                                              T_lim_coldside, phase_coldside,
+                                              H_lim_coldside, heating=False)
     
     # Pinch on the hot side
     Q_cold_stream = heat_exchange_to_condition(s_cold_in, s_cold_out, 
-                                               T_lim_hotside, phase_hotside)
+                                               T_lim_hotside, phase_hotside,
+                                               H_lim_hotside, heating=True)
+    
+    if Q_hot_stream == Q_cold_stream == 0.:
+        s0_out.copy_like(s0_in)
+        s1_in.copy_like(s1_out)
+        return 0.
     
     if Q_hot_stream > 0 or Q_cold_stream < 0:
         # Sanity check
+        if Q_hot_stream / s_hot_in.C < 0.1 or Q_cold_stream / s_cold_in.C > -0.1:
+            s0_out.copy_like(s0_in)
+            s1_in.copy_like(s1_out)
+            return 0.
         raise RuntimeError('inlet stream not in vapor-liquid equilibrium')
     
     if Q_cold_stream < -Q_hot_stream:
@@ -118,8 +189,8 @@ def counter_current_heat_exchange(s0_in, s1_in, s0_out, s1_out,
 def heuristic_overall_heat_transfer_coefficient(ci, hi, co, ho):
     """
     Return a heuristic estimate of the overall heat transfer coefficient
-    [U; in kW/m^2/K]. Assume `U` is 0.5 kW/m^2/K if heat exchange is sensible 
-    and 1.0 kW/m^2/K otherwise.
+    [U; in kW/m^2/K]. Assume `U` is 1.0 kW/m^2/K if heat exchange is between
+    a condensing fluid and a vaporizing fluid and 1.0 kW/m^2/K otherwise.
     
     Parameters
     ----------
@@ -169,12 +240,12 @@ def heuristic_pressure_drop(inlet_phase, outlet_phase):
     if ('l' in inlet_phase and 'g' in outlet_phase) or ('g' in inlet_phase and 'l' in outlet_phase):
         # Latent fluid (boiling or condensing)
         dP = 1.5
-    elif inlet_phase == 'l':
+    elif 'l' in inlet_phase:
         # Sensible liquid
-        dP = 5
-    elif outlet_phase == 'g':
+        dP = 5.
+    elif 'g' in outlet_phase:
         # Sensible vapor
-        dP = 3
+        dP = 3.
     return dP
 
 def heuristic_tubeside_and_shellside_pressure_drops(ci, hi, co, ho,
@@ -249,7 +320,7 @@ def order_streams(in_a, in_b, out_a, out_b):
 
 # %% Computational functions
 
-@njitable(cache=True)
+@njit(cache=True)
 def compute_fallback_Fahkeri_LMTD_correction_factor(P, N_shells):
     """Return LMTF correction factor using the fallback equation for
     `compute_Fahkeri_LMTD_correction_factor` when logarithms cannot be computed."""
@@ -267,7 +338,7 @@ def compute_fallback_Fahkeri_LMTD_correction_factor(P, N_shells):
             Ft = (2**0.5*J)/ln(K)
     return Ft
 
-@njitable(cache=True)
+@njit(cache=True)
 def compute_Fahkeri_LMTD_correction_factor(Tci, Thi, Tco, Tho, N_shells):
     r"""
     Return the log-mean temperature difference correction factor `Ft` 
@@ -357,7 +428,7 @@ def compute_Fahkeri_LMTD_correction_factor(Tci, Thi, Tco, Tho, N_shells):
         Ft = 0.5
     return Ft
 
-@njitable(cache=True)
+@njit(cache=True)
 def compute_heat_transfer_area(LMTD, U, Q, ft):
     """
     Return required heat transfer area by LMTD correction factor method.
@@ -374,7 +445,7 @@ def compute_heat_transfer_area(LMTD, U, Q, ft):
     """
     return Q/(U*LMTD*ft)   
 
-@njitable(cache=True)
+@njit(cache=True)
 def compute_LMTD(Thi, Tho, Tci, Tco, counterflow=True):
     r'''
     Return the log-mean temperature difference of an ideal counterflow
@@ -431,5 +502,5 @@ def compute_LMTD(Thi, Tho, Tci, Tco, counterflow=True):
     if abs(dTF21) < 1e-8:
         LMTD = dTF1
     else:
-        LMTD = dTF21/ln(dTF2/dTF1)
+        LMTD = dTF21/ln(dTF2 / dTF1)
     return LMTD
\ No newline at end of file
