@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -10,7 +10,7 @@
 from .specification_factors import (pressure_vessel_material_factors,
                                     material_densities_lb_per_ft3)
 from . import flash_vessel_design as design
-from ...utils import bounds_warning
+from ...exceptions import bounds_warning, DesignWarning
 from warnings import warn
 
 __all__ = ('PressureVessel',)
@@ -33,8 +33,9 @@ class PressureVessel:
                'Vertical vessel length': (12, 40)}
     
     # Bare module factors
-    _BM = {'Horizontal pressure vessel': 3.05,
-           'Vertical pressure vessel': 4.16}
+    _F_BM_default = {'Horizontal pressure vessel': 3.05,
+                     'Vertical pressure vessel': 4.16,
+                     'Platform and ladders': 1.}
     
     @property
     def vessel_type(self):
@@ -55,7 +56,9 @@ class PressureVessel:
         return self._vessel_material
     @vessel_material.setter
     def vessel_material(self, material):
-        try: self._F_M = pressure_vessel_material_factors[material]
+        try: 
+            F_M = self.F_M
+            F_M['Vertical pressure vessel'] = F_M['Horizontal pressure vessel'] = pressure_vessel_material_factors[material]
         except KeyError:
             raise ValueError(f"no material factor available for '{material}'; "
                               "only the following materials are available: "
@@ -79,12 +82,15 @@ class PressureVessel:
         return method(pressure, diameter, length)
     
     def _horizontal_vessel_design(self, pressure, diameter, length) -> dict:
+        pressure = pressure
+        diameter = diameter
+        length = length
         # Calculate vessel weight and wall thickness
         rho_M = material_densities_lb_per_ft3[self._vessel_material]
         if pressure < 14.68:
             warn('vacuum pressure vessel ASME codes not implemented yet; '
                  'wall thickness may be inaccurate and stiffening rings may be '
-                 'required')
+                 'required', category=DesignWarning)
         VW, VWT = design.compute_vessel_weight_and_wall_thickness(
             pressure, diameter, length, rho_M)
         bounds_warning(self, 'Horizontal vessel weight', VW, 'lb',
@@ -100,11 +106,14 @@ class PressureVessel:
         return Design
     
     def _vertical_vessel_design(self, pressure, diameter, length) -> dict:
+        pressure = pressure
+        diameter = diameter
+        length = length
         rho_M = material_densities_lb_per_ft3[self._vessel_material]
         if pressure < 14.68:
             warn('vacuum pressure vessel ASME codes not implemented yet; '
                  'wall thickness may be inaccurate and stiffening rings may be '
-                 'required')
+                 'required', category=DesignWarning)
         VW, VWT = design.compute_vessel_weight_and_wall_thickness(
             pressure, diameter, length, rho_M)
         Design = {}
@@ -122,6 +131,9 @@ class PressureVessel:
         return Design
 
     def _vessel_purchase_cost(self, weight, diameter, length) -> dict:
+        weight = weight
+        diameter = diameter
+        length = length
         vessel_type = self.vessel_type
         if vessel_type == 'Horizontal':
             method = self._horizontal_vessel_purchase_cost
@@ -132,7 +144,9 @@ class PressureVessel:
         return method(weight, diameter, length)
 
     def _horizontal_vessel_purchase_cost(self, weight, diameter, length=None) -> dict:
-        return {'Horizontal pressure vessel': design.compute_horizontal_vessel_purchase_cost(weight, diameter, self._F_M)}
+        return {'Horizontal pressure vessel': design.compute_horizontal_vessel_purchase_cost(weight),
+                'Platform and ladders': design.compute_horizontal_vessel_platform_and_ladders_purchase_cost(diameter)}
 
     def _vertical_vessel_purchase_cost(self, weight, diameter, length) -> dict:
-        return {'Vertical pressure vessel': design.compute_vertical_vessel_purchase_cost(weight, diameter, length, self._F_M)}
\ No newline at end of file
+        return {'Vertical pressure vessel': design.compute_vertical_vessel_purchase_cost(weight),
+                'Platform and ladders': design.compute_vertical_vessel_platform_and_ladders_purchase_cost(diameter, length)}
\ No newline at end of file
