@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -21,13 +21,13 @@ References
 """
 import biosteam as bst
 from math import ceil
-from thermosteam import settings
+from warnings import warn
 from thermosteam.units_of_measure import AbsoluteUnitsOfMeasure
 from ...utils import ExponentialFunctor
 
 __all__ = ('TankPurchaseCostAlgorithm',
            'field_erected_tank_purchase_cost',
-           'compute_number_of_tanks_and_total_purchase_cost',
+           'compute_number_of_tanks_and_purchase_cost',
            'storage_tank_purchase_cost_algorithms',
            'mix_tank_purchase_cost_algorithms')
 
@@ -82,14 +82,14 @@ class TankPurchaseCostAlgorithm:
         self.material = material
 
     def __repr__(self):
-        return f"{type(self).__name__}(f_Cp={self.f_Cp.__name__}, V_min={self.V_min}, V_max={self.V_max}, CE={self.CE}, material={self.material}, V_units={self.V_units})"
+        f_name = getattr(self.f_Cp, '__name__', str(self.f_Cp))
+        return f"{type(self).__name__}(f_Cp={f_name}, V_min={self.V_min}, V_max={self.V_max}, CE={self.CE}, material={self.material}, V_units={self.V_units})"
     
     
-def compute_number_of_tanks_and_total_purchase_cost(total_volume,
-                                                    purchase_cost_algorithm,
-                                                    material_factor):
+def compute_number_of_tanks_and_purchase_cost(total_volume,
+                                                    purchase_cost_algorithm):
     """
-    Return number of tanks and total purchase cost of all tanks.
+    Return number of tanks and purchase cost of a single tank.
 
     Parameters
     ----------
@@ -97,25 +97,21 @@ def compute_number_of_tanks_and_total_purchase_cost(total_volume,
         Total volume required [m^3].
     purchase_cost_algorithm : TankPurchaseCostAlgorithm
         All costing options.
-    material_factor : float
-        Material purchase cost factor.
     
     """
     V_total = total_volume
-    F_M = material_factor
     V_units = purchase_cost_algorithm.V_units
     V_total /= V_units.conversion_factor('m^3')
     V_min = purchase_cost_algorithm.V_min
-    if settings.debug and V_total < V_min:
-        raise RuntimeError(
-             f"volume ({V_total:.5g} {V_units}) is below "
+    if V_total < V_min:
+        warn(f"volume ({V_total:.5g} {V_units}) is below "
              f"the lower bound ({V_min:.5g} {V_units}) for purchase "
-              "cost estimation")
+              "cost estimation", bst.exceptions.CostWarning)
     N = ceil(V_total / purchase_cost_algorithm.V_max)
     if N:
         V = V_total / N
         F_CE = bst.CE / purchase_cost_algorithm.CE
-        Cp = N * F_M * F_CE * purchase_cost_algorithm.f_Cp(V)
+        Cp = F_CE * purchase_cost_algorithm.f_Cp(V)
     else:
         Cp = 0.
     return N, Cp
