@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -10,8 +10,6 @@ This module contains heat exchanger unit operations.
 
 .. contents:: :local:
     
-Unit Operations
----------------
 .. autoclass:: biosteam.units.heat_exchange.HX
 .. autoclass:: biosteam.units.heat_exchange.HXutility
 .. autoclass:: biosteam.units.heat_exchange.HXprocess 
@@ -22,14 +20,17 @@ from .._graphics import utility_heat_exchanger_graphics, process_heat_exchanger_
 from .design_tools.specification_factors import (
     shell_and_tube_material_factor_coefficients,
     compute_shell_and_tube_material_factor)
+from ..utils import list_available_names
 from .design_tools import heat_transfer as ht
 import numpy as np
 import biosteam as bst
 from math import exp, log as ln
+from typing import Optional
+from .._heat_utility import UtilityAgent
 
 __all__ = ('HX', 'HXutility', 'HXprocess')
 
-# Lenght factor 
+# Length factor 
 x = np.array((8, 13, 16, 20)) 
 y = np.array((1.25, 1.12,1.05,1))
 p2 = np.polyfit(x, y, 2)
@@ -71,7 +72,7 @@ class HX(Unit, isabstract=True):
         heat.
 
     """
-    line = 'Heat Exchanger'
+    line = 'Heat exchanger'
     _units = {'Area': 'ft^2',
               'Overall heat transfer coefficient': 'kW/m^2/K',
               'Log-mean temperature difference': 'K',
@@ -81,24 +82,23 @@ class HX(Unit, isabstract=True):
               'Total tube length': 'ft'}
     _N_ins = 1
     _N_outs = 1
-    _N_heat_utilities = 1
-    _BM = {'Double pipe': 1.8,
-           'Floating head': 3.17,
-           'Fixed head': 3.17,
-           'U tube': 3.17,
-           'Kettle vaporizer': 3.17}
+    _F_BM_default = {'Double pipe': 1.8,
+                     'Floating head': 3.17,
+                     'Fixed head': 3.17,
+                     'U tube': 3.17,
+                     'Kettle vaporizer': 3.17}
     
     @property
     def material(self):
         """Default 'Carbon steel/carbon steel'"""
-        return self.material
+        return self._material
     @material.setter
     def material(self, material):
         try:
             self._F_Mab = shell_and_tube_material_factor_coefficients[material]
         except KeyError:
             raise AttributeError("material must be one of the following: "
-                                 f"{', '.join(shell_and_tube_material_factor_coefficients)}")
+                                 f"{list_available_names(shell_and_tube_material_factor_coefficients)}")
         self._material = material  
     
     @property
@@ -111,12 +111,20 @@ class HX(Unit, isabstract=True):
             self._Cb_func = Cb_dict[heat_exchanger_type]
         except KeyError:
             raise AttributeError("heat exchange type must be one of the following: "
-                                 f"{', '.join(Cb_dict)}")
+                                 f"{list_available_names(Cb_dict)}")
         self._heat_exchanger_type = heat_exchanger_type     
 
+    def _assert_compatible_property_package(self):
+        if self.owner is not self: return
+        assert all([i.chemicals is j.chemicals for i, j in zip(self._ins, self._outs) if (i and j)]), (
+            "inlet and outlet stream chemicals are incompatible; "
+            "try using the `thermo` keyword argument to initialize the unit operation "
+            "with a compatible thermodynamic property package"
+        )
+
     def _design(self):
-        # Get duty (kW)
-        Q = abs(self.Q) / 3600
+        # Get heat transfer (kW)
+        Q = abs(self.total_heat_transfer) / 3600
         
         if Q <= 1e-12: 
             self.design_results.clear()
@@ -162,9 +170,7 @@ class HX(Unit, isabstract=True):
 
     def _cost(self):
         Design = self.design_results
-        if not Design: 
-            self.purchase_costs.clear()
-            return
+        if not Design: return
             
         A = Design['Area']
         L = Design['Total tube length']
@@ -192,7 +198,10 @@ class HX(Unit, isabstract=True):
             C_b = self._Cb_func(A, bst.CE)
         
         # Free on board purchase prize
-        self.purchase_costs[heat_exchanger_type] = F_p * F_l * F_m * C_b
+        self.F_M[heat_exchanger_type] = F_m
+        self.F_P[heat_exchanger_type] = F_p
+        self.F_D[heat_exchanger_type] = F_l
+        self.baseline_purchase_costs[heat_exchanger_type] = C_b
 
 
 class HXutility(HX):
@@ -202,9 +211,9 @@ class HXutility(HX):
 
     Parameters
     ----------
-    ins : stream
+    ins : 
         Inlet.
-    outs : stream
+    outs : 
         Outlet.
     T=None : float
         Temperature of outlet stream [K].
@@ -216,11 +225,19 @@ class HXutility(HX):
         Enforced overall heat transfer coefficent [kW/m^2/K].
     heat_exchanger_type : str, optional
         Heat exchanger type. Defaults to "Floating head".
-    N_shells=2 : int
-        Number of shells.
-    ft=None : float
+    N_shells : int, optional
+        Number of shells. Defaults to 2.
+    ft : float, optional
         User imposed correction factor.
-    
+    heat_only : bool, optional
+        If True, heat exchanger can only heat.
+    cool_only : bool, optional
+        If True, heat exchanger can only cool.
+    heat_transfer_efficiency : bool, optional
+        User enforced heat transfer efficiency. A value less than 1
+        means that a fraction of heat transfered is lost to the environment.
+        Defaults to the heat transfer efficiency of the utility agent.
+        
     Notes
     -----
     Must specify either `T` or `V` when creating a HXutility object.
@@ -249,11 +266,11 @@ class HXutility(HX):
         flow (kmol/hr): Water    200
                         Ethanol  200
     >>> hx.results()
-    Heat Exchanger                                            Units       hx
+    Heat exchanger                                            Units       hx
     Low pressure steam  Duty                                  kJ/hr 1.01e+06
-                        Flow                                kmol/hr     26.1
-                        Cost                                 USD/hr     6.21
-    Design              Area                                   ft^2       57
+                        Flow                                kmol/hr     26.2
+                        Cost                                 USD/hr     6.22
+    Design              Area                                   ft^2     59.9
                         Overall heat transfer coefficient  kW/m^2/K      0.5
                         Log-mean temperature difference           K      101
                         Fouling correction factor                          1
@@ -261,9 +278,9 @@ class HXutility(HX):
                         Shell side pressure drop                psi        5
                         Operating pressure                      psi       50
                         Total tube length                        ft       20
-    Purchase cost       Double pipe                             USD 4.74e+03
-    Total purchase cost                                         USD 4.74e+03
-    Utility cost                                             USD/hr     6.21
+    Purchase cost       Double pipe                             USD 4.78e+03
+    Total purchase cost                                         USD 4.78e+03
+    Utility cost                                             USD/hr     6.22
     
     Run heat exchanger by vapor fraction:
     
@@ -280,14 +297,35 @@ class HXutility(HX):
                         Ethanol  200
     outs...
     [0] product
-        phase: 'g', T: 357.45 K, P: 101325 Pa
+        phase: 'g', T: 357.44 K, P: 101325 Pa
         flow (kmol/hr): Water    200
                         Ethanol  200
     >>> hx.results()
-    Heat Exchanger                                            Units       hx
+    Heat exchanger                                            Units       hx
     Low pressure steam  Duty                                  kJ/hr 1.94e+07
-                        Flow                                kmol/hr      499
+                        Flow                                kmol/hr      500
                         Cost                                 USD/hr      119
+    Design              Area                                   ft^2      716
+                        Overall heat transfer coefficient  kW/m^2/K        1
+                        Log-mean temperature difference           K     80.8
+                        Fouling correction factor                          1
+                        Tube side pressure drop                 psi      1.5
+                        Shell side pressure drop                psi      1.5
+                        Operating pressure                      psi       50
+                        Total tube length                        ft       20
+    Purchase cost       Floating head                           USD 2.65e+04
+    Total purchase cost                                         USD 2.65e+04
+    Utility cost                                             USD/hr      119
+
+    We can also specify the heat transfer efficiency of the heat exchanger:
+        
+    >>> hx.heat_transfer_efficiency = 1. # Originally 0.95 for low pressure steam
+    >>> hx.simulate()
+    >>> hx.results() # Notice how the duty, utility cost, and capital cost decreased
+    Heat exchanger                                            Units       hx
+    Low pressure steam  Duty                                  kJ/hr 1.84e+07
+                        Flow                                kmol/hr      475
+                        Cost                                 USD/hr      113
     Design              Area                                   ft^2      680
                         Overall heat transfer coefficient  kW/m^2/K        1
                         Log-mean temperature difference           K     80.8
@@ -298,21 +336,82 @@ class HXutility(HX):
                         Total tube length                        ft       20
     Purchase cost       Floating head                           USD 2.61e+04
     Total purchase cost                                         USD 2.61e+04
-    Utility cost                                             USD/hr      119
+    Utility cost                                             USD/hr      113
     
+    Run heat exchanger by vapor fraction:
+    
+    >>> feed = Stream('feed', Water=200, Ethanol=200)
+    >>> hx = HXutility('hx', ins=feed, outs='product', V=1,
+    ...                rigorous=True) # Include VLE
+    >>> hx.simulate()
+    >>> hx.show()
+    HXutility: hx
+    ins...
+    [0] feed
+        phase: 'l', T: 298.15 K, P: 101325 Pa
+        flow (kmol/hr): Water    200
+                        Ethanol  200
+    outs...
+    [0] product
+        phase: 'g', T: 357.44 K, P: 101325 Pa
+        flow (kmol/hr): Water    200
+                        Ethanol  200
+    >>> hx.results()
+    Heat exchanger                                            Units       hx
+    Low pressure steam  Duty                                  kJ/hr 1.94e+07
+                        Flow                                kmol/hr      500
+                        Cost                                 USD/hr      119
+    Design              Area                                   ft^2      716
+                        Overall heat transfer coefficient  kW/m^2/K        1
+                        Log-mean temperature difference           K     80.8
+                        Fouling correction factor                          1
+                        Tube side pressure drop                 psi      1.5
+                        Shell side pressure drop                psi      1.5
+                        Operating pressure                      psi       50
+                        Total tube length                        ft       20
+    Purchase cost       Floating head                           USD 2.65e+04
+    Total purchase cost                                         USD 2.65e+04
+    Utility cost                                             USD/hr      119
+
+    We can also specify the heat transfer efficiency of the heat exchanger:
+        
+    >>> hx.heat_transfer_efficiency = 1. # Originally 0.95 for low pressure steam
+    >>> hx.simulate()
+    >>> hx.results() # Notice how the duty, utility cost, and capital cost decreased
+    Heat exchanger                                            Units       hx
+    Low pressure steam  Duty                                  kJ/hr 1.84e+07
+                        Flow                                kmol/hr      475
+                        Cost                                 USD/hr      113
+    Design              Area                                   ft^2      680
+                        Overall heat transfer coefficient  kW/m^2/K        1
+                        Log-mean temperature difference           K     80.8
+                        Fouling correction factor                          1
+                        Tube side pressure drop                 psi      1.5
+                        Shell side pressure drop                psi      1.5
+                        Operating pressure                      psi       50
+                        Total tube length                        ft       20
+    Purchase cost       Floating head                           USD 2.61e+04
+    Total purchase cost                                         USD 2.61e+04
+    Utility cost                                             USD/hr      113
+
     """
-    line = 'Heat Exchanger'
+    line = 'Heat exchanger'
     _graphics = utility_heat_exchanger_graphics
     
     def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
-                 T=None, V=None, rigorous=False, U=None,
-                 heat_exchanger_type="Floating head",
-                 material="Carbon steel/carbon steel",
-                 N_shells=2,
-                 ft=None):
+            T=None, V=None, rigorous=False, U=None, H=None,
+            heat_exchanger_type="Floating head",
+            material="Carbon steel/carbon steel",
+            N_shells=2,
+            ft=None,
+            heat_only=None,
+            cool_only=None,
+            heat_transfer_efficiency=None,
+        ):
         super().__init__(ID, ins, outs, thermo)
-        self.T = T #: Temperature of outlet stream (K).
-        self.V = V #: Vapor fraction of outlet stream.
+        self.T = T #: [float] Temperature of outlet stream (K).
+        self.V = V #: [float] Vapor fraction of outlet stream.
+        self.H = H #: [float] Enthalpy of outlet stream.
         
         #: [bool] If true, calculate vapor liquid equilibrium
         self.rigorous = rigorous
@@ -320,90 +419,158 @@ class HXutility(HX):
         #: [float] Enforced overall heat transfer coefficent (kW/m^2/K)
         self.U = U
         
-        #: [float] Total heat transfered.
-        self.Q = None
-        
-        #: Number of shells for LMTD correction factor method.
+        #: [int] Number of shells for LMTD correction factor method.
         self.N_shells = N_shells
         
-        #: User imposed correction factor.
+        #: [float] User imposed correction factor.
         self.ft = ft
+
+        #: [bool] If True, heat exchanger can only heat. 
+        self.heat_only = heat_only
+        
+        #: [bool] If True, heat exchanger can only cool. 
+        self.cool_only = cool_only
         
         self.material = material
         self.heat_exchanger_type = heat_exchanger_type
-    
-    def _init_utils(self):
-        # tuple[HeatUtility] All heat utilities associated to unit
-        self._heat_utilities = tuple([bst.HeatUtility(heat_exchanger=self)
-                                      for i in range(self._N_heat_utilities)])
         
-        # [PowerUtility] Electric utility associated to unit
-        self.power_utility = bst.PowerUtility()
+        #: [bool] User enforced heat transfer efficiency. A value less than 1
+        #: means that a fraction of heat transfered is lost to the environment.
+        #: If value is None, it defaults to the heat transfer efficiency of the 
+        #: heat utility.
+        self.heat_transfer_efficiency = heat_transfer_efficiency
     
     @property
-    def heat_utilities(self):
-        return self._heat_utilities
-    @heat_utilities.setter
-    def heat_utilities(self, heat_utilities):
-        self._heat_utilities = heat_utilities = tuple(heat_utilities)
-        for i in heat_utilities: i.heat_exchanger = self
+    def total_heat_transfer(self):
+        """[float] Heat transfer in kJ/hr, including environmental losses."""
+        return abs(self.net_duty)
+    Q = total_heat_transfer # Alias for backward compatibility
     
-    def simulate_as_auxiliary_exchanger(self, duty, stream):
-        self.outs[0] = stream.proxy()
-        self.ins[0] = stream.proxy()
-        hu = self.heat_utilities[0]
-        hu.heat_exchanger = None
-        hu(duty, stream.T)
-        self.Q = duty
-        super()._design()
-        self._cost()
+    def simulate_as_auxiliary_exchanger(self, 
+            ins, outs=None, duty=None, vle=True, scale=None, hxn_ok=True, P=None,
+        ):
+        inlet = self.ins[0]
+        outlet = self.outs[0]
+        if not inlet: inlet = inlet.materialize_connection(None)
+        if not outlet: outlet = outlet.materialize_connection(None)
+        idata = inlet.get_data()
+        inlet.mix_from(ins, energy_balance=False)
+        if P is None: P = inlet.P
+        if vle: inlet.vle(H=sum([i.H for i in ins]), P=P)
+        if outs is None:
+            if duty is None: raise ValueError('must pass duty when no outlets are given')
+            outlet.copy_like(inlet)
+            if vle: 
+                outlet.vle(H=inlet.H + duty, P=P)
+            else:
+                outlet.Hnet = inlet.Hnet + duty
+        else:
+            outlet.mix_from(outs)
+            outlet.P = P
+            if duty is None: 
+                duty = outlet.Hnet - inlet.Hnet
+            elif vle: 
+                outlet.vle(H=inlet.H + duty, P=P)
+            else:
+                outlet.Hnet = inlet.Hnet + duty
+        if scale is not None:
+            duty *= scale
+            inlet.scale(scale)
+            outlet.scale(scale)
+        self.simulate(
+            run=False, # Do not run mass and energy balance
+            design_kwargs=dict(duty=duty),
+        )
+        for i in self.heat_utilities: i.hxn_ok = hxn_ok
+        inlet.set_data(idata)
         
     def _run(self):
         feed = self.ins[0]
-        s = self.outs[0]
-        s.copy_flow(feed)
-        s.P = feed.P
+        outlet = self.outs[0]
+        outlet.copy_flow(feed)
+        if outlet.isempty(): return
+        outlet.P = feed.P
         T = self.T
         V = self.V
+        H = self.H
+        T_given = T is not None
         V_given = V is not None
-        if self.rigorous:
-            if T and V_given:
-                raise RuntimeError("may only define either temperature, 'T', or "
-                                   "vapor fraction 'V', in a rigorous simulation")
+        H_given = H is not None
+        N_given = T_given + V_given + H_given
+        if N_given == 0:
+            raise RuntimeError("no specification available; must define at either "
+                               "temperature 'T', vapor fraction, 'V', or enthalpy 'H'")
+        elif self.rigorous:
+            if N_given > 1:
+                raise RuntimeError("may only specify either temperature, 'T', "
+                                   "vapor fraction 'V', or enthalpy 'H', "
+                                   "in a rigorous simulation")
             if V_given:
                 if V == 0:
-                    s.phase = 'l'
-                    s.T = s.bubble_point_at_P().T
+                    outlet.phase = 'l'
+                    outlet.T = outlet.bubble_point_at_P().T
                 elif V == 1:
-                    s.phase = 'g'
-                    s.T = s.dew_point_at_P().T
+                    outlet.phase = 'g'
+                    outlet.T = outlet.dew_point_at_P().T
                 elif 0 < V < 1:
-                    s.vle(V=V, P=s.P)
+                    outlet.vle(V=V, P=outlet.P)
                 else:
                     raise RuntimeError("vapor fraction, 'V', must be a "
                                        "positive fraction")
+            elif T_given:
+                if outlet.isempty():
+                    outlet.T = T
+                else:
+                    try:
+                        outlet.vle(T=T, P=outlet.P)
+                    except RuntimeError as e:
+                        if len(outlet.phases) > 1: raise e
+                        T_bubble = outlet.bubble_point_at_P().T
+                        if T <= T_bubble:
+                            outlet.phase = 'l'
+                        else:
+                            T_dew = outlet.dew_point_at_P().T
+                            if T_dew >= T:
+                                outlet.phase = 'g'
+                            else:
+                                raise RuntimeError('outlet in vapor-liquid equilibrium, but stream is linked')
+                        outlet.T = T
+                    except ValueError:
+                        outlet.vle(T=T, P=outlet.P)
             else:
-                s.vle(T=T, P=s.P)
-        elif (T or V_given):
+                outlet.vle(H=H, P=outlet.P)
+        else:
+            if T_given and H_given:
+                raise RuntimeError("cannot specify both temperature, 'T' "
+                                   "and enthalpy 'H'")
+            if T_given:
+                outlet.T = T
+            else:
+                outlet.T = feed.T
             if V_given:
                 if V == 0:
-                    s.phase = 'l'
+                    outlet.phase = 'l'
                 elif V == 1:
-                    s.phase = 'g'
+                    outlet.phase = 'g'
                 else:
                     raise RuntimeError("vapor fraction, 'V', must be either "
                                        "0 or 1 in a non-rigorous simulation")
+                if V == 1 and feed.vapor_fraction < 1. and (outlet.T + 1e-6) < feed.T:
+                    raise ValueError('outlet cannot be cooler than inlet if boiling')
+                if V == 0 and feed.vapor_fraction > 0. and outlet.T > feed.T + 1e-6:
+                    raise ValueError('outlet cannot be hotter than inlet if condensing')
             else:
                 phase = feed.phase
-                if len(phase) == 1: s.phase = phase
-            if T:
-                s.T = T
-            else:
-                s.T = feed.T
-        else:
-            raise RuntimeError("must define either temperature, 'T', and/or "
-                               "vapor fraction, 'V', in a non-rigorous "
-                               "simulation")
+                if len(phase) == 1: outlet.phase = phase
+            if H_given:
+                outlet.H = H
+                
+        if self.heat_only and outlet.H - feed.H < 0.: 
+            outlet.copy_like(feed)
+            return
+        if self.cool_only and outlet.H - feed.H > 0.: 
+            outlet.copy_like(feed)
+            return
 
     def get_streams(self):
         """
@@ -430,10 +597,19 @@ class HXutility(HX):
 
     def _design(self, duty=None):
         # Set duty and run heat utility
-        if duty is None:
-            duty = self.H_out - self.H_in
-        self.heat_utilities[0](duty, self.ins[0].T, self.outs[0].T)
-        self.Q = duty
+        if duty is None: duty = self.Hnet # Includes heat of formation
+        inlet = self.ins[0]
+        outlet = self.outs[0] 
+        T_in = inlet.T
+        T_out = outlet.T
+        iscooling = duty < 0.
+        if iscooling: # Assume there is a pressure drop before the heat exchanger
+            if T_out > T_in: T_in = T_out
+        else:
+            if T_out < T_in: T_out = T_in
+        self.add_heat_utility(duty, T_in, T_out, 
+                              heat_transfer_efficiency=self.heat_transfer_efficiency,
+                              hxn_ok=True)
         super()._design()
 
 
@@ -444,10 +620,10 @@ class HXprocess(HX):
     
     Parameters
     ----------
-    ins : stream sequence
+    ins : 
         * [0] Inlet process fluid a
         * [1] Inlet process fluid b  
-    outs : stream sequence
+    outs : 
         * [0] Outlet process fluid a        
         * [1] Outlet process fluid b
     U=None : float, optional
@@ -468,6 +644,10 @@ class HXprocess(HX):
         User enforced phase of outlet stream at index 0.
     phase1=None : 'l' or 'g', optional
         User enforced phase of outlet stream at index 1.
+    H_lim0 : float, optional
+        Enthalpy limit of outlet stream at index 0.
+    H_lim1 : float, optional
+        Enthalpy limit of outlet stream at index 1.
     
     Examples
     --------
@@ -480,26 +660,26 @@ class HXprocess(HX):
     >>> in_b = Stream('in_b', Water=200)
     >>> hx = HXprocess('hx', ins=(in_a, in_b), outs=('out_a', 'out_b'))
     >>> hx.simulate()
-    >>> hx.show()
+    >>> hx.show(T='degC:.2g')
     HXprocess: hx
     ins...
     [0] in_a
-        phase: 'g', T: 351.43 K, P: 101325 Pa
+        phase: 'g', T: 78 degC, P: 101325 Pa
         flow (kmol/hr): Ethanol  50
     [1] in_b
-        phase: 'l', T: 298.15 K, P: 101325 Pa
+        phase: 'l', T: 25 degC, P: 101325 Pa
         flow (kmol/hr): Water  200
     outs...
     [0] out_a
-        phases: ('g', 'l'), T: 351.39 K, P: 101325 Pa
-        flow (kmol/hr): (g) Ethanol  31.3
-                        (l) Ethanol  18.7
+        phases: ('g', 'l'), T: 78 degC, P: 101325 Pa
+        flow (kmol/hr): (g) Ethanol  31.4
+                        (l) Ethanol  18.6
     [1] out_b
-        phases: ('g', 'l'), T: 346.43 K, P: 101325 Pa
-        flow (kmol/hr): (l) Water  200
+        phase: 'l', T: 73 degC, P: 101325 Pa
+        flow (kmol/hr): Water  200
     
     >>> hx.results()
-    Heat Exchanger                                            Units       hx
+    Heat exchanger                                            Units       hx
     Design              Area                                   ft^2      213
                         Overall heat transfer coefficient  kW/m^2/K      0.5
                         Log-mean temperature difference           K     20.4
@@ -537,13 +717,26 @@ class HXprocess(HX):
         phase: 'l', T: 303.15 K, P: 101325 Pa
         flow (kmol/hr): Water  200
     [1] out_b
-        phase: 'l', T: 327.92 K, P: 101325 Pa
+        phase: 'l', T: 328.09 K, P: 101325 Pa
         flow (kmol/hr): Ethanol  200
     >>> hx.results()
-    Heat Exchanger                                            Units       hx
-    Design              Area                                   ft^2      367
+    Heat exchanger                                            Units       hx
+    Design              Area                                   ft^2      369
                         Overall heat transfer coefficient  kW/m^2/K      0.5
-                        Log-mean temperature difference           K     11.5
+                        Log-mean temperature difference           K     11.4
+                        Fouling correction factor                          1
+                        Tube side pressure drop                 psi        5
+                        Shell side pressure drop                psi        5
+                        Operating pressure                      psi     14.7
+                        Total tube length                        ft       20
+    Purchase cost       Floating head                           USD 2.23e+04
+    Total purchase cost                                         USD 2.23e+04
+    Utility cost                                             USD/hr        0
+    >>> hx.results()
+    Heat exchanger                                            Units       hx
+    Design              Area                                   ft^2      369
+                        Overall heat transfer coefficient  kW/m^2/K      0.5
+                        Log-mean temperature difference           K     11.4
                         Fouling correction factor                          1
                         Tube side pressure drop                 psi        5
                         Shell side pressure drop                psi        5
@@ -554,9 +747,8 @@ class HXprocess(HX):
     Utility cost                                             USD/hr        0
     
     """
-    line = 'Heat Exchanger'
+    line = 'Heat exchanger'
     _graphics = process_heat_exchanger_graphics
-    _N_heat_utilities = 0
     _N_ins = 2
     _N_outs = 2
     
@@ -566,14 +758,17 @@ class HXprocess(HX):
                  heat_exchanger_type="Floating head",
                  N_shells=2, ft=None, 
                  phase0=None,
-                 phase1=None):
+                 phase1=None,
+                 H_lim0=None,
+                 H_lim1=None,
+        ):
         super().__init__(ID, ins, outs, thermo)
         
         #: [float] Enforced overall heat transfer coefficent (kW/m^2/K)
         self.U = U
         
-        #: [float] Total heat transfered.
-        self.Q = None
+        #: [float] Total heat transfered in kJ/hr (not including losses).
+        self.total_heat_transfer = None
         
         #: Number of shells for LMTD correction factor method.
         self.N_shells = N_shells
@@ -590,27 +785,41 @@ class HXprocess(HX):
         #: [float] Temperature limit of outlet stream at index 1.
         self.T_lim1 = T_lim1
         
-        #: Enforced phase of outlet at index 0
+        #: [float] Temperature limit of outlet stream at index 0.
+        self.H_lim0 = H_lim0
+        
+        #: [float] Temperature limit of outlet stream at index 1.
+        self.H_lim1 = H_lim1
+        
+        #: [str] Enforced phase of outlet at index 0
         self.phase0 = phase0
         
-        #: Enforced phase of outlet at index 1
+        #: [str] Enforced phase of outlet at index 1
         self.phase1 = phase1
         
         self.material = material
         self.heat_exchanger_type = heat_exchanger_type
+        self.reset_streams_at_setup = False
         
     def get_streams(self):
         s_in_a, s_in_b = self.ins
         s_out_a, s_out_b  = self.outs
         return s_in_a, s_in_b, s_out_a, s_out_b
     
+    @property
+    def Q(self): 
+        # Alias for backwards compatibility, not documented to avoid users
+        # from depending on it.
+        return self.total_heat_transfer
+    @Q.setter
+    def Q(self, Q):
+        self.total_heat_transfer = Q
+    
     def _setup(self):
-        for i in self._ins:
-            if i.source: i.empty()
-            
-    def simulate(self):
-        self._run()
-        self._summary()
+        super()._setup()
+        if self.reset_streams_at_setup:
+            for i in self._ins:
+                if i.source: i.empty()
     
     def _run(self):
         s1_in, s2_in = self._ins
@@ -618,17 +827,22 @@ class HXprocess(HX):
         if s1_in.isempty():
             s1_out.empty()
             s2_out.copy_like(s2_in)
-            self.Q = 0.
+            self.total_heat_transfer = 0.
         elif s2_in.isempty():
             s2_out.empty()
             s1_out.copy_like(s1_in)
-            self.Q = 0.
+            self.total_heat_transfer = 0.
         else:
             s1_out.copy_like(s1_in)
             s2_out.copy_like(s2_in)
             self._run_counter_current_heat_exchange()
+            for s_out in (s1_out, s2_out):
+                if isinstance(s_out, bst.MultiStream):
+                    phase = s_out.phase
+                    if len(phase) == 1: s_out.phase = phase
     
     def _run_counter_current_heat_exchange(self):
-        self.Q = ht.counter_current_heat_exchange(*self._ins, *self._outs,
-                                                  self.dT, self.T_lim0, self.T_lim1,
-                                                  self.phase0, self.phase1)
+        self.total_heat_transfer = ht.counter_current_heat_exchange(
+            *self._ins, *self._outs, self.dT, self.T_lim0, self.T_lim1,
+            self.phase0, self.phase1, self.H_lim0, self.H_lim1
+        )
