@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -11,39 +11,39 @@ from thermosteam import utils
 from . import (misc,
                patches,
                tictoc,
-               not_implemented_method,
+               abstract_method,
                piping,
                stream_link_options,
-               unit_warnings,
                functors,
                stream_filters,
+               scope,
 )
 __all__ = ('colors',
            'misc', 
            'patches', 
            'tictoc',
-           'not_implemented_method',
+           'abstract_method',
            'functors',
            *utils.__all__,
            *patches.__all__, 
-           *not_implemented_method.__all__, 
+           *abstract_method.__all__, 
            *misc.__all__, 
            *tictoc.__all__, 
            *piping.__all__, 
            *stream_link_options.__all__,
-           *unit_warnings.__all__,
            *functors.__all__,
            *stream_filters.__all__,
+           *scope.__all__,
 )
 from thermosteam.utils import *
-from .not_implemented_method import *
+from .abstract_method import *
 from .misc import *
 from .patches import *
 from .tictoc import *
 from .piping import *
 from .stream_link_options import *
-from .unit_warnings import *
 from .functors import *
 from .stream_filters import *
+from .scope import *
 
 del utils
\ No newline at end of file
