@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -11,10 +11,14 @@ This module includes arbitrary classes and functions.
 """
 import biosteam as bst
 from thermosteam import Chemicals
+from math import ceil
 
 __all__ = ('factor', 'checkbounds', 'strtuple',
-           'format_unit_line', 'format_unit_name',
-           'remove_undefined_chemicals')
+           'format_title', 'format_unit_name',
+           'remove_undefined_chemicals',
+           'default_chemical_dict', 'subgroup',
+           'repr_subgroups', 'repr_items',
+           'list_available_names')
 
 # %% Number functions
 
@@ -27,6 +31,11 @@ def checkbounds(x, bounds):
 
 # %% String functions
 
+def list_available_names(names):
+    *names, last = list(names)
+    names = ', '.join([repr(i) for i in names]) + ', and ' + repr(last)
+    return names
+
 def strtuple(iterable):
     """Return string of all items in the tuple""" 
     string = ''
@@ -40,16 +49,18 @@ def strtuple(iterable):
     string = '(' + string + ')'
     return string
         
-def format_unit_line(line):
+def format_title(line):
     line = line.replace('_', ' ')
     words = []
     word = ''
+    last = ''
     for i in line:
-        if i.isupper():
+        if i.isupper() and last.isalpha() and not last.isupper():
             words.append(word)
             word = i
         else:
             word += i
+        last = i
     words.append(word)
     line = ''
     for word in words:
@@ -62,21 +73,45 @@ def format_unit_line(line):
     first_word, *rest = line.split(' ')
     words = [first_word[0].capitalize() + first_word[1:]]
     for word in rest:
-        if not all([i.isupper() for i in word]):
+        if not all([(i.isupper() or not last.isalpha()) for i in word]):
             word = word.lower()
         words.append(word)
     return ' '.join(words)
 
 def format_unit_name(name):
-    words = name.split(' ')
-    new_words = []
-    for i in words:
-        new_words.append(i[0].capitalize() + i[1:])
-    return ''.join(new_words)
+    return ''.join([i[0].capitalize() + i[1:] for i in name.split(' ')])
     
+def subgroup(items, size=5):
+    return [items[size*i: size*(i+1)] for i in range(int(ceil(len(items) / size)))]
+
+def repr_subgroups(subgroups):
+    return [', '.join([str(i) for i in j]) for j in subgroups]
+
+def repr_items(start, items, subgroup_size=5, brackets=None):
+    N_spaces = len(start)
+    if brackets:
+        left, right = brackets
+        N_spaces += 1
+    else:
+        left = ''; right = ''
+    subgroups = repr_subgroups(subgroup(items, subgroup_size))
+    dlim = ",\n" + " " * N_spaces
+    return start + left + dlim.join(subgroups) + right
 
 # %% Chemical management
 
 def remove_undefined_chemicals(data: dict, chemicals: Chemicals):
     for i in tuple(data):
         if i not in chemicals: del data[i]
+
+def default_chemical_dict(dct, chemicals, g, l, s, n=None):
+    if n is None: n = l
+    for i in chemicals:
+        ID = i.ID
+        if ID not in dct:
+            locked_state = i.locked_state
+            if locked_state == 'g': dct[ID] = g
+            elif locked_state == 'l': dct[ID] = l
+            elif locked_state == 's': dct[ID] = s
+            elif locked_state is None: dct[ID] = n
+            else: raise RuntimeError(f"unknown locked state '{locked_state}' of chemical '{i}'")
